@@ -1,7 +1,8 @@
 import logging
+from typing import Dict, cast
 from grpc import StatusCode
 import grpc
-from grpc.aio import ServerInterceptor, AioRpcError
+from grpc.aio import ServerInterceptor, ServicerContext, Metadata
 from prometheus_client import (
     Counter,
     Histogram,
@@ -22,6 +23,35 @@ REGISTRY.unregister(PROCESS_COLLECTOR)
 REGISTRY.unregister(PLATFORM_COLLECTOR)
 REGISTRY.unregister(REGISTRY._names_to_collectors["python_gc_objects_collected_total"])
 
+"""As ServicerContext.code() returns the integer value of the StatusCode we
+want to get the enum value for it"""
+status_code_by_int_value: Dict[int, grpc.StatusCode] = {
+    cast(int, enum_field.value[0]): enum_field for enum_field in grpc.StatusCode
+}
+
+
+class ServicerContextProxy:
+    """This class proxy extends the ServicerContext.abort method to correctly
+    set the code the rpc was aborted with.
+
+    This is a known issue with grpc python: https://github.com/grpc/grpc/issues/27409
+    """
+
+    def __init__(self, servicer_context: ServicerContext):
+        self.__dict__["servicer_context"] = servicer_context
+
+    def __setattr__(self, key, value):
+        setattr(self.servicer_context, key, value)
+
+    def __getattr__(self, key):
+        return getattr(self.__dict__["servicer_context"], key)
+
+    async def abort(
+        self, code: grpc.StatusCode, details: str = "", trailing_metadata: Metadata = tuple()  # type: ignore
+    ):
+        self.set_code(code)  # type: ignore
+        return await self.servicer_context.abort(code, details, trailing_metadata)
+
 
 class MetricsInterceptor(ServerInterceptor):
     registry: CollectorRegistry
@@ -123,7 +153,8 @@ class MetricsInterceptor(ServerInterceptor):
                             grpc_method=grpc_method_name,
                         ).inc()
                         response_or_iterator = await behavior(
-                            request_or_iterator, servicer_context
+                            request_or_iterator,
+                            ServicerContextProxy(servicer_context),
                         )
                         grpc_code = StatusCode.OK
                         self.delphai_response_count.labels(
@@ -133,9 +164,11 @@ class MetricsInterceptor(ServerInterceptor):
                             grpc_code=grpc_code,
                         ).inc()
                         return response_or_iterator
-                    except AioRpcError as ex:
-                        grpc_code = ex.code()
-                        logging.error(f"[{ex.code()}] {ex.details()}")
+                    except grpc.aio.AbortError as ex:
+                        grpc_code = status_code_by_int_value.get(
+                            servicer_context.code(), StatusCode.UNKNOWN
+                        )
+                        logging.error(f"[{grpc_code}] {ex.details()}")
                         self.delphai_response_count.labels(
                             grpc_type=grpc_type,
                             grpc_service=grpc_service_name,
